package com.sysdeo.eclipse.tomcat;

/*
 * (c) Copyright Sysdeo SA 2001, 2002.
 * All Rights Reserved.
 */

import java.util.ArrayList;

import org.eclipse.debug.internal.ui.preferences.ComboFieldEditor;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.IVMInstallType;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import com.sysdeo.eclipse.tomcat.editors.ClasspathFieldEditor;
import com.sysdeo.eclipse.tomcat.editors.ListFieldEditor;


public class TomcatJVMPreferencePage extends PreferencePage implements IWorkbenchPreferencePage, TomcatPluginResources {

	static final private int FIELD_WIDTH = 50;
	private ComboFieldEditor jvmChoice;
	private ListFieldEditor jvmParamaters;
	private ClasspathFieldEditor jvmClasspath;
	private ClasspathFieldEditor jvmBootClasspath;
	
	public TomcatJVMPreferencePage() {
		super();
		setPreferenceStore(TomcatLauncherPlugin.getDefault().getPreferenceStore());
//		setDescription("");
	}

	/*
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout();
		gl.numColumns = 1;
		composite.setLayout(gl);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessVerticalSpace = true;
		gd.grabExcessHorizontalSpace = true;
		composite.setLayoutData(gd);
				
		Composite group1 = new Composite(composite,SWT.NONE);
		GridLayout gl2 = new GridLayout(3,false);
		group1.setLayout(gl2);
		gd = new GridData();
		gd.horizontalAlignment = gd.FILL;
		group1.setLayoutData(gd);		

		// Collect all JREs
		ArrayList allVMs = new ArrayList();
		IVMInstallType[] vmTypes = JavaRuntime.getVMInstallTypes();
		for (int i = 0; i < vmTypes.length; i++) {
			IVMInstall[] vms = vmTypes[i].getVMInstalls();
			for (int j = 0; j < vms.length; j++) {
				allVMs.add(vms[j]);
			}
		}
		
		String[][] namesAndValues = new String[allVMs.size()][2];
		for (int i=0; i<allVMs.size(); i++) {
			namesAndValues[i][0] = ((IVMInstall)allVMs.get(i)).getName();
			namesAndValues[i][1] = ((IVMInstall)allVMs.get(i)).getId();			
		}
	
		jvmChoice = new ComboFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_JRE_KEY,
				PREF_PAGE_JRE_LABEL,
				namesAndValues,
				group1);
		
		jvmParamaters = new ListFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_JVM_PARAMETERS_KEY,
				PREF_PAGE_PARAMETERS_LABEL,
				composite);	
						
		jvmClasspath = new ClasspathFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_JVM_CLASSPATH_KEY,
				PREF_PAGE_CLASSPATH_LABEL,
				composite);

		jvmBootClasspath = new ClasspathFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_JVM_BOOTCLASSPATH_KEY,
				PREF_PAGE_BOOTCLASSPATH_LABEL,
				composite);

		this.initField(jvmChoice);
		this.initField(jvmParamaters);
		this.initField(jvmClasspath);
		this.initField(jvmBootClasspath);

		return composite;
	}

	/*
	 * @see IWorkbenchPreferencePage#init(IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}


	public boolean performOk() {
		jvmChoice.store();	
		jvmBootClasspath.store();
		jvmClasspath.store();
		jvmParamaters.store();
		TomcatLauncherPlugin.getDefault().savePluginPreferences();		
		return true;	
	}
	
	private void initField(FieldEditor field) {
		field.setPreferenceStore(getPreferenceStore());
		field.setPreferencePage(this);
		field.load();		
	}

}

