package com.sysdeo.eclipse.tomcat;

/*
 * (c) Copyright Sysdeo SA 2001, 2002.
 * All Rights Reserved.
 */

import java.io.File;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import com.sysdeo.eclipse.tomcat.editors.ProjectListEditor;


public class TomcatPreferencePage extends PreferencePage implements IWorkbenchPreferencePage, TomcatPluginResources {

	static final private int FIELD_WIDTH = 50;
	private RadioGroupFieldEditor version;
	private DirectoryFieldEditor home;
	private FileFieldEditor configFile;
	private ProjectListEditor projectListEditor;
	private BooleanFieldEditor debugModeEditor;
	private ModifyListener fModifyListener;
		
	public TomcatPreferencePage() {
		super();
		setPreferenceStore(TomcatLauncherPlugin.getDefault().getPreferenceStore());
//		setDescription("");

	}

	/*
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());

		Composite versionGroup = new Composite(composite,SWT.NONE);
		version = new RadioGroupFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_VERSION_KEY,
				PREF_PAGE_CHOOSEVERSION_LABEL,
				1,
				new String[][] {
					{PREF_PAGE_VERSION3_LABEL, TomcatLauncherPlugin.TOMCAT_VERSION3},
					{PREF_PAGE_VERSION4_LABEL, TomcatLauncherPlugin.TOMCAT_VERSION4}, 
					{PREF_PAGE_VERSION4_1_LABEL, TomcatLauncherPlugin.TOMCAT_VERSION41},
					{PREF_PAGE_VERSION5_LABEL, TomcatLauncherPlugin.TOMCAT_VERSION5}  	
				},
				versionGroup);
				
		new Label(composite, SWT.NULL); //blank
		
		Composite homeGroup = new Composite(composite,SWT.NONE);
		GridData gd = new GridData();
		gd.horizontalAlignment = gd.FILL;
		homeGroup.setLayoutData(gd);		
		home = new DirectoryFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_HOME_KEY,
				PREF_PAGE_HOME_LABEL,
				homeGroup);
				
		configFile = new FileFieldEditor(
				TomcatLauncherPlugin.TOMCAT_PREF_CONFIGFILE_KEY,
				PREF_PAGE_CONFIGFILE_LABEL,
				homeGroup);
					
		home.setPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if(event.getProperty().equals(home.VALUE)) {		
					computeConfigFile();
				}
			}
		});
			
		new Label(composite, SWT.NULL); //blank
		
		projectListEditor = new ProjectListEditor();
		Control projectList = projectListEditor.getControl(composite);
		GridData gd2 = new GridData();
		gd2.horizontalAlignment = gd2.FILL;
		projectList.setLayoutData(gd2);		

		this.initField(version);
		this.initField(home);
		this.initField(configFile);
		if(configFile.getStringValue().length() == 0) {
			computeConfigFile();	
		}
		projectListEditor.setCheckedElements(TomcatLauncherPlugin.getDefault().getProjectsInCP());		 

		new Label(composite, SWT.NULL); //blank

		Composite debugModeGroup = new Composite(composite,SWT.NONE);
		debugModeEditor = new BooleanFieldEditor(
			TomcatLauncherPlugin.TOMCAT_PREF_DEBUGMODE_KEY,
			PREF_PAGE_DEBUGMODE_LABEL,
			debugModeGroup);
		this.initField(debugModeEditor);
		
		return composite;
	}

	/*
	 * @see IWorkbenchPreferencePage#init(IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}


	public boolean performOk() {	
		version.store();
		home.store();
		TomcatLauncherPlugin.getDefault().initTomcatClasspathVariable();
		configFile.store();
		debugModeEditor.store();
		TomcatLauncherPlugin.getDefault().setProjectsInCP(projectListEditor.getCheckedElements());
		TomcatLauncherPlugin.getDefault().savePluginPreferences();
		return true;	
	}
	
	private void initField(FieldEditor field) {
		field.setPreferenceStore(getPreferenceStore());
		field.setPreferencePage(this);
		field.load();		
	}
	
	private void computeConfigFile() {
		configFile.setStringValue(home.getStringValue() + File.separator + "conf" + File.separator + "server.xml");	
	}
	
}

