package com.sysdeo.eclipse.tomcat;

/*
 * (c) Copyright Sysdeo SA 2001, 2002.
 * All Rights Reserved.
 */

import java.io.File;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class TomcatProjectGeneralPropertyPage implements TomcatPluginResources {

	private Button isTomcatProjectCheck;
	private Button updateXmlCheck;
	private Button exportSourceCheck;
	private Button reloadableCheck;
	private Button redirectLoggerCheck;
	private Text webpathText;
	private Text warLocationText;
	private Text rootDirText;
	private TomcatProjectPropertyPage page;
	
	private static final int TEXT_FIELD_WIDTH = 200;
	
	public TomcatProjectGeneralPropertyPage(TomcatProjectPropertyPage page) {
		this.page = page;
	}
	
	
	/**
	 * returns a control which consists of the ui elements of this page
	 */
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
	
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		createIsTomcatProjectGroup(composite);
//		new Label(composite, SWT.NULL); //blank

		createWebpathGroup(composite);
//		new Label(composite, SWT.NULL); //blank

		createWarLocationGroup(composite);
	
		createRootDirGroup(composite);
		
		return composite;
	}

	public void createIsTomcatProjectGroup(Composite parent) {
		Composite isTomcatProjectGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		isTomcatProjectGroup.setLayout(layout);
		isTomcatProjectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// project location entry field
		isTomcatProjectCheck = new Button(isTomcatProjectGroup, SWT.CHECK | SWT.LEFT);
		isTomcatProjectCheck.setText(PROPERTIES_PAGE_PROJECT_ISTOMCATPROJECT_LABEL);
		isTomcatProjectCheck.setEnabled(true);

		try {		
			isTomcatProjectCheck.setSelection(page.getJavaProject().getProject().hasNature(TomcatLauncherPlugin.NATURE_ID));
		} catch (CoreException ex) {
			TomcatLauncherPlugin.log(ex.getMessage());	
		}
	}

	public void createWebpathGroup(Composite parent) {
		Composite webpathGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		webpathGroup.setLayout(layout);
		webpathGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// location label
		Label webpathLabel = new Label(webpathGroup,SWT.NONE);
		webpathLabel.setText(WIZARD_PROJECT_WEBPATH_LABEL);
		webpathLabel.setEnabled(true);

		// project location entry field
		webpathText = new Text(webpathGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
//		data.horizontalSpan = 2;
		webpathText.setLayoutData(data);		
		webpathText.setText(this.getWebPath()); 
		webpathText.setEnabled(true);
						
		// project location entry field
		updateXmlCheck = new Button(webpathGroup, SWT.CHECK | SWT.LEFT);
		updateXmlCheck.setText(WIZARD_PROJECT_UPDATEXML_LABEL);
		data = new GridData();
		data.horizontalSpan = 3;
		updateXmlCheck.setLayoutData(data);	
		updateXmlCheck.setEnabled(true);
		updateXmlCheck.setSelection(this.getUpdateXml());
		
		// reloadable attribute			
		reloadableCheck = new Button(webpathGroup,SWT.CHECK | SWT.LEFT);		
		reloadableCheck.setText(WIZARD_PROJECT_RELOADABLE_LABEL);
		data = new GridData();
		data.horizontalSpan = 3;		
		reloadableCheck.setLayoutData(data);
		reloadableCheck.setEnabled(true);
		reloadableCheck.setSelection(this.getReloadable());				

		// reloadable attribute			
		redirectLoggerCheck = new Button(webpathGroup,SWT.CHECK | SWT.LEFT);		
		redirectLoggerCheck.setText(WIZARD_PROJECT_REDIRECTLOGGER_LABEL);
		data = new GridData();
		data.horizontalSpan = 3;		
		redirectLoggerCheck.setLayoutData(data);
		redirectLoggerCheck.setEnabled(true);
		redirectLoggerCheck.setSelection(this.getRedirectLogger());
	}
	
	public void createWarLocationGroup(Composite parent) {
		Composite warLocationGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		warLocationGroup.setLayout(layout);
		warLocationGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// location label
		Label warLocationLabel = new Label(warLocationGroup,SWT.NONE);
		warLocationLabel.setText(WIZARD_PROJECT_WARLOCATION_LABEL);
		warLocationLabel.setEnabled(true);

		// project location entry field
		warLocationText = new Text(warLocationGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
		warLocationText.setLayoutData(data);		
		warLocationText.setText(this.getWarLocation()); 
		warLocationText.setEnabled(true);
		
		Button browseButton = new Button(warLocationGroup, SWT.PUSH);
		browseButton.setText(BROWSE_BUTTON_LABEL);
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent evt) {
				String newValue = warFieldChange();
				if (newValue != null) {
					warLocationText.setText(newValue);
				}
			}
		});

		browseButton.setEnabled(true);
		
		exportSourceCheck = new Button(warLocationGroup, SWT.CHECK | SWT.LEFT);
		exportSourceCheck.setText(WIZARD_PROJECT_EXPORTSOURCE_LABEL);
		data = new GridData();
		data.horizontalSpan = 3;
		exportSourceCheck.setLayoutData(data);		
		exportSourceCheck.setEnabled(true);
		exportSourceCheck.setSelection(this.getExportSource());
	}

	public void createRootDirGroup(Composite parent) {
		Composite rootDirGroup = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		rootDirGroup.setLayout(layout);
		rootDirGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// location label
		Label rootDirLabel = new Label(rootDirGroup,SWT.NONE);
		rootDirLabel.setText(WIZARD_PROJECT_ROOTDIR_LABEL);
		rootDirLabel.setEnabled(true);

		// project location entry field
		rootDirText = new Text(rootDirGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
		data.horizontalSpan = 3;
		rootDirText.setLayoutData(data);
		rootDirText.setText(this.getRootDir());
		rootDirText.setEnabled(true);
	}
			
	protected String getWebPath() {
		String result = "";
		try {
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getWebPath();
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}
	
	protected String getWarLocation() {
		String result = "";
		try {
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getWarLocation();
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}

	
	protected String getRootDir() {
		String result = "/";
		try {
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getRootDir();
		} catch (CoreException ex) {
			// result = "";
		}
		return result;
	}

	protected boolean getUpdateXml() {
		boolean result = true;
		try {
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getUpdateXml();
		} catch (CoreException ex) {
			// result = false;
		}
		return result;
	}

	protected boolean getExportSource() {
		boolean result = false;
		try {
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getExportSource();
		} catch (CoreException ex) {
			// result = false;
		}
		return result;
	}

	protected boolean getReloadable(){
		boolean result = true;
		try{
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getReloadable();
		}catch(CoreException ex){
		}
		return result;
	}

	protected boolean getRedirectLogger(){
		boolean result = false;
		try{
			TomcatProject prj = page.getTomcatProject();
			if(prj != null)
				result = prj.getRedirectLogger();
		}catch(CoreException ex){
		}
		return result;
	}
				
	/**
	 * performes the ok action for this property page
	 */
	public boolean performOk() {
		try {
			if(isTomcatProjectCheck.getSelection()) {		
				TomcatProject.addTomcatNature(page.getJavaProject());
				TomcatProject prj = page.getTomcatProject();
				prj.updateWebPath(webpathText.getText());
				prj.setWarLocation(warLocationText.getText());
				prj.updateWebPath(webpathText.getText());				
				prj.setUpdateXml(updateXmlCheck.getSelection());
				prj.setExportSource(exportSourceCheck.getSelection());
				prj.setReloadable(reloadableCheck.getSelection());				
				prj.setRedirectLogger(redirectLoggerCheck.getSelection());
				prj.setRootDir(rootDirText.getText());
				prj.saveProperties();
			} else {
				page.getTomcatProject().removeContextInServerXML();
				TomcatProject.removeTomcatNature(page.getJavaProject());
			}
		} catch (Exception ex) {
			TomcatLauncherPlugin.log(ex.getMessage());	
		}
		
		return true;
	}
	
	
	protected String warFieldChange() {
		File f = new File(warLocationText.getText());
		if (!f.exists())
			f = null;
		File d = getFile(f);
		if (d == null)
			return null;
	
		return d.getAbsolutePath();
	}
	
	public boolean isTomcatProjectChecked() {
		return isTomcatProjectCheck.getSelection();
	}
	
	/**
	 * Helper to open the file chooser dialog.
	 */
	private File getFile(File startingDirectory) {
	
		FileDialog dialog = new FileDialog(page.getShell(), SWT.OPEN);
		if (startingDirectory != null)
			dialog.setFileName(startingDirectory.getPath());
	//	if (extensions != null)
	//		dialog.setFilterExtensions(extensions);
		String file = dialog.open();
		if (file != null) {
			file = file.trim();
			if (file.length() > 0)
				return new File(file);
		}
	
		return null;
	}

}
